/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include <iostream>
#include <string>
#include <map>
#include <stdlib.h>

#include "Font.h"
#include "Library.h"

using namespace std;

/*
 * ライブラリ初期化
 */
Uint32 SDL_FLAGS  = SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER;
Uint32 VIDEO_FLAGS = SDL_HWSURFACE|SDL_SRCALPHA|SDL_ANYFORMAT;
Uint32 SURFACE_FLAGS = SDL_HWSURFACE|SDL_SRCALPHA;
int BPP = 32;

TTF_Font* systemFont = NULL;
SDL_Color systemColor = { 255,255,255,0 };

map<string, SDLKey> Library::keyboardMap;

bool Library::initialized = false;
void Library::init()
{
    if ( initialized ) {
        cout << "Library::init(): invalid call." << endl;
        return;
    }

    // SDL初期化
    if ( SDL_Init(SDL_FLAGS) == -1 ) {
        cerr << "SDL_Init: " << SDL_GetError() << endl;
        exit(-1);
    }


    // ウィンドウの仮設定を行う
    SDL_SetVideoMode(1, 1, 32, SDL_HWSURFACE);

    // マウスカーソルを表示に設定
    SDL_ShowCursor(SDL_DISABLE);

    // SDL_ttfの初期化
    if ( TTF_Init() == -1 ) {
        cerr << "TTF_Init: " << TTF_GetError() << endl;
        exit(-1);
    }

    if ( SDL_WasInit(SDL_INIT_AUDIO) == 0 ) {
        SDL_InitSubSystem(SDL_INIT_AUDIO);
    }

    // SDL_mixerの初期化
    if ( Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 256) == -1 ) {
        cerr << "Mix_OpenAudio: " << Mix_GetError() << endl;
        exit(-1);
    }
    Mix_AllocateChannels(3);

    // キーボードマッピング
    keyboardMap["上"] = SDLK_UP;
    keyboardMap["下"] = SDLK_DOWN;
    keyboardMap["左"] = SDLK_LEFT;
    keyboardMap["右"] = SDLK_RIGHT;
    keyboardMap["スペース"] = SDLK_SPACE;
    keyboardMap["改行"] = SDLK_RETURN;
    keyboardMap["バックスペース"] = SDLK_BACKSPACE;
    keyboardMap["タブ"] = SDLK_TAB;
    keyboardMap["エスケープ"] = SDLK_ESCAPE;
    keyboardMap["ダラー"] = SDLK_DOLLAR;
    keyboardMap["アンパーサンド"] = SDLK_AMPERSAND;
    keyboardMap["左かっこ"] = SDLK_LEFTPAREN;
    keyboardMap["右かっこ"] = SDLK_RIGHTPAREN;
    keyboardMap["アスタリスク"] = SDLK_ASTERISK;
    keyboardMap["プラス"] = SDLK_PLUS;
    keyboardMap["カンマ"] = SDLK_COMMA;
    keyboardMap["マイナス"] = SDLK_MINUS;
    keyboardMap["ピリオド"] = SDLK_PERIOD;
    keyboardMap["スラッシュ"] = SDLK_SLASH;
    keyboardMap["０"] = SDLK_0;
    keyboardMap["１"] = SDLK_1;
    keyboardMap["２"] = SDLK_2;
    keyboardMap["３"] = SDLK_3;
    keyboardMap["４"] = SDLK_4;
    keyboardMap["５"] = SDLK_5;
    keyboardMap["６"] = SDLK_6;
    keyboardMap["７"] = SDLK_7;
    keyboardMap["８"] = SDLK_8;
    keyboardMap["９"] = SDLK_9;
    keyboardMap["コロン"] = SDLK_COLON;
    keyboardMap["セミコロン"] = SDLK_SEMICOLON;
    keyboardMap["小なり"] = SDLK_LESS;
    keyboardMap["等号"] = SDLK_EQUALS;
    keyboardMap["大なり"] = SDLK_GREATER;
    keyboardMap["クエスチョンマーク"] = SDLK_QUESTION;
    keyboardMap["＠"] = SDLK_AT;
    keyboardMap["左ブラケット"] = SDLK_LEFTBRACKET;
    keyboardMap["右ブラケット"] = SDLK_RIGHTBRACKET;
    keyboardMap["キャレット"] = SDLK_CARET;
    keyboardMap["アンダースコア"] = SDLK_UNDERSCORE;
    keyboardMap["バッククォート"] = SDLK_BACKQUOTE;
    keyboardMap["バックスラッシュ"] = SDLK_BACKSLASH;
    keyboardMap["a"] = SDLK_a;
    keyboardMap["b"] = SDLK_b;
    keyboardMap["c"] = SDLK_c;
    keyboardMap["d"] = SDLK_d;
    keyboardMap["e"] = SDLK_e;
    keyboardMap["f"] = SDLK_f;
    keyboardMap["g"] = SDLK_g;
    keyboardMap["h"] = SDLK_h;
    keyboardMap["i"] = SDLK_i;
    keyboardMap["j"] = SDLK_j;
    keyboardMap["k"] = SDLK_k;
    keyboardMap["l"] = SDLK_l;
    keyboardMap["m"] = SDLK_m;
    keyboardMap["o"] = SDLK_o;
    keyboardMap["p"] = SDLK_p;
    keyboardMap["q"] = SDLK_q;
    keyboardMap["r"] = SDLK_r;
    keyboardMap["s"] = SDLK_s;
    keyboardMap["t"] = SDLK_t;
    keyboardMap["u"] = SDLK_u;
    keyboardMap["v"] = SDLK_v;
    keyboardMap["w"] = SDLK_w;
    keyboardMap["x"] = SDLK_x;
    keyboardMap["y"] = SDLK_y;
    keyboardMap["z"] = SDLK_z;
    keyboardMap["削除"] = SDLK_DELETE;

    keyboardMap["テンキー０"] = SDLK_KP0;
    keyboardMap["テンキー１"] = SDLK_KP1;
    keyboardMap["テンキー２"] = SDLK_KP2;
    keyboardMap["テンキー３"] = SDLK_KP3;
    keyboardMap["テンキー４"] = SDLK_KP4;
    keyboardMap["テンキー５"] = SDLK_KP5;
    keyboardMap["テンキー６"] = SDLK_KP6;
    keyboardMap["テンキー７"] = SDLK_KP7;
    keyboardMap["テンキー８"] = SDLK_KP8;
    keyboardMap["テンキー９"] = SDLK_KP9;
    keyboardMap["テンキーピリオド"] = SDLK_KP_PERIOD;
    keyboardMap["テンキー割り算"] = SDLK_KP_DIVIDE;
    keyboardMap["テンキーかけ算"] = SDLK_KP_MULTIPLY;
    keyboardMap["テンキーマイナス"] = SDLK_KP_MINUS;
    keyboardMap["テンキープラス"] = SDLK_KP_PLUS;
    keyboardMap["テンキーエンター"] = SDLK_KP_ENTER;
    keyboardMap["テンキーイコール"] = SDLK_KP_EQUALS;

    keyboardMap["ホーム"] = SDLK_HOME;
    keyboardMap["エンド"] = SDLK_END;
    keyboardMap["ページアップ"] = SDLK_PAGEUP;
    keyboardMap["ページダウン"] = SDLK_PAGEDOWN;

    keyboardMap["ファンクションキー１"] = SDLK_F1;
    keyboardMap["ファンクションキー２"] = SDLK_F2;
    keyboardMap["ファンクションキー３"] = SDLK_F3;
    keyboardMap["ファンクションキー４"] = SDLK_F4;
    keyboardMap["ファンクションキー５"] = SDLK_F5;
    keyboardMap["ファンクションキー６"] = SDLK_F6;
    keyboardMap["ファンクションキー７"] = SDLK_F7;
    keyboardMap["ファンクションキー８"] = SDLK_F8;
    keyboardMap["ファンクションキー９"] = SDLK_F9;
    keyboardMap["ファンクションキー１０"] = SDLK_F10;
    keyboardMap["ファンクションキー１１"] = SDLK_F11;
    keyboardMap["ファンクションキー１２"] = SDLK_F12;
    keyboardMap["ファンクションキー１３"] = SDLK_F13;
    keyboardMap["ファンクションキー１４"] = SDLK_F14;
    keyboardMap["ファンクションキー１５"] = SDLK_F15;


    // システムフォント初期化
    {
        if ( TTF_Init() == -1 ) {
            cerr << "TTF_Init: " << TTF_GetError() << endl;
            exit(-1);
        }

        const string fontPath[] = {
            "c:/winnt/fonts/msgothic.ttc",
            "c:/windows/fonts/msgothic.ttc",
            "/usr/X11R6/lib/X11/fonts/truetype/みかちゃん.ttf",
            "/usr/X11/lib/X11/fonts/truetype/みかちゃん.ttf",
            "/usr/share/fonts/TrueType/mika.ttf",
            "/usr/X11R6/lib/X11/fonts/TrueType/mika.ttf",
            "/usr/share/fonts/truetype/みかちゃん.ttf",

            "/usr/X11R6/lib/X11/fonts/truetype/sazanami-gothic.ttf",
            "/usr/X11/lib/X11/fonts/truetype/sazanami-gothic.ttf",
            "/usr/share/fonts/TrueType/sazanami-gothic.ttf",
            "/usr/X11R6/lib/X11/fonts/TrueType/sazanami-gothic.ttf",
            "/usr/share/fonts/truetype/sazanami-gothic.ttf",
            "/usr/share/fonts/TrueType/FS-Gothic-gs.ttf",
            "/usr/X11R6/lib/X11/fonts/TrueType/FS-Gothic.ttf",
            "/usr/share/fonts/TrueType/gt200001.ttf",
            "/usr/X11R6/lib/X11/fonts/TrueType/gt200001.ttf",
            ""
        };


        for ( int i=0; !fontPath[i].empty(); i++ ) {
            systemFont = TTF_OpenFont(fontPath[i].c_str(), 12);
            if ( !systemFont ) { continue; }
            else { break; }
        }
    }

   // 初期化完了
    initialized = true;

    // 終了処理を登録
    atexit(Library::end);
}

void Library::end()
{
    if ( !initialized ) { 
        cout << "Library::end(): Invalid call." << endl;
        return;
    }
    freeImages();
    Mix_CloseAudio();
    TTF_Quit();
    SDL_Quit();
}

SDL_Surface* Library::createWindow(Uint32 width, Uint32 height)
{
    SDL_Surface* surface = SDL_SetVideoMode(width, height, BPP, VIDEO_FLAGS);
    if ( surface == NULL ) {
        cerr << "SDL_SetVideoMode: " << SDL_GetError() << endl;
        exit(-1);
    }

    return surface;
}

SDL_Surface* Library::createFilledString(const char* text, const SDL_Color& fg, const SDL_Color& bg)
{
    if ( !systemFont ) {
        return SDL_CreateRGBSurface(NULL,1,1,1,0,0,0,0);
    }
    return TTF_RenderUTF8_Shaded(
        systemFont,
        text,
        fg,
        bg
    );
}

SDL_Surface* Library::createString(const char* text, const SDL_Color* color)
{
    if ( !systemFont ) {
        return SDL_CreateRGBSurface(NULL,1,1,1,0,0,0,0);
    }
    return TTF_RenderUTF8_Solid(
        systemFont,
        text,
        *color
    );
}

SDL_Surface* Library::createString(const char* text)
{
    if ( !systemFont ) {
        return SDL_CreateRGBSurface(NULL,1,1,1,0,0,0,0);
    }
    return TTF_RenderUTF8_Solid(
        systemFont,
        text,
        systemColor
    );
}



SDL_Surface* Library::createString(const char* text, const char* fontPath, int size, const SDL_Color& color, int style)
{
    if ( *fontPath == 0 ) {
        return TTF_RenderUTF8_Blended(getSystemFont(), text, color);
    }

    TTF_Font* font;
    font = TTF_OpenFont(fontPath, size);
    if ( !font ) {
        cerr << "Library::createString(text,fontpath,size,color,style);" << endl;
        cerr << "フォントを開けませんでした。" << endl;
        cerr << TTF_GetError() << endl;
        return SDL_CreateRGBSurface(NULL,1,1,1,0,0,0,0);
    }
    TTF_SetFontStyle(font, style);
    SDL_Surface* surface = TTF_RenderUTF8_Blended(font, text, color);
    TTF_CloseFont(font);
    return surface;
}

map<string, SDL_Surface*> Library::images;

SDL_Surface* Library::loadImage(const string& file)
{
    map<string, SDL_Surface*>::iterator s = images.find(file);
    if ( s != images.end() ) { return (*s).second; }

    SDL_Surface* surface_ = IMG_Load(file.c_str());
    if ( surface_ == NULL ) {
        cerr << "IMG_Load(\"" << file << "\"): " << IMG_GetError() << endl;
        exit(-1);
    }
    SDL_SetAlpha(surface_, SDL_HWSURFACE|SDL_SRCALPHA, 255);
    SDL_Surface* surface = SDL_DisplayFormatAlpha(surface_);
    if ( surface == NULL ) {
        cerr << "SDL_DisplayFormatAlpha(\"" << file << "\"): " << SDL_GetError() << endl;
        exit(-1);
    }
 
    SDL_FreeSurface(surface_);

    images[file] = surface;

    return surface;
}

void Library::freeImages()
{
    map<string, SDL_Surface*>::iterator s = images.begin();
    while ( s != images.end() ) {
        SDL_FreeSurface( (*s).second );
        s++;
    }
    images.clear();
}

SDLKey Library::getKeySym(string name)
{
    if ( keyboardMap.find(name) == keyboardMap.end() ) { return SDLK_UNKNOWN; }
    return keyboardMap[name];
}

TTF_Font* Library::getSystemFont()
{
    return systemFont;
}
