/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#include <SDL/SDL.h>
#include <math.h>
#include <typeinfo>

#include "Face.h"
#include "GC.h"
#include "Text.h"
#include "Library.h"
#include "Scene/SceneMediator.h"

TextEvent::Event* TextEvent::Font::clone() {
    TextEvent::Font* e = new TextEvent::Font();
    e->fontPath = fontPath;
    return e;
}

TextEvent::Event* TextEvent::LocationX::clone() {
    TextEvent::LocationX* e = new TextEvent::LocationX();
    e->x = x;
    return e;
}
TextEvent::Event* TextEvent::LocationY::clone() {
    TextEvent::LocationY* e = new TextEvent::LocationY();
    e->y = y;
    return e;
}

TextEvent::Event* TextEvent::Color::clone() {
    TextEvent::Color* e = new TextEvent::Color();
    e->color = color;
    return e;
}

TextEvent::Event* TextEvent::Style::clone() {
    TextEvent::Style* e = new TextEvent::Style();
    e->style = style;
    return e;
}

TextEvent::Event* TextEvent::Size::clone() {
    TextEvent::Size* e = new TextEvent::Size();
    e->size = size;
    return e;
}

TextEvent::Event* TextEvent::Wait::clone() {
    TextEvent::Wait* e = new TextEvent::Wait();
    e->count = count;
    return e;
}

TextEvent::Event* TextEvent::String::clone() {
    TextEvent::String* e = new TextEvent::String();
    e->str = str;
    return e;
}

Text::Text()
{
}

Text::~Text()
{
}

Text* Text::create(list<TextEvent::Event*>* events)
{
    Text* text = GC<Text>::getInstance();
    text->x = 0;
    text->y = 0;
    text->timerCount = 0;
    text->size = 12;
    SDL_Color default_color = { 0,0,0,0 };
    text->color = default_color;
    text->style = NULL;
    text->fontPath = "";
    if ( events == NULL ) {
        text->events = new list<TextEvent::Event*>;
    }
    else {
        text->events = events;
    }
    text->faces.clear();
    return text;
}

bool Text::action(SceneMediator* mediator)
{
    if ( timerCount-- > 0 ) { return true; }
    if( timerCount-- <= 0 && events_itr == events->end() ) {
        list<Face*>::iterator i = faces.begin();
        while ( i !=faces.end() ) {
            mediator->removeFace(*i);
            i++;
        }
        destroy();
        return false;
    }

    if ( typeid(**events_itr)==typeid(TextEvent::LocationX) ) {
        x = ((TextEvent::LocationX*)(*events_itr))->x;
    }
    else if ( typeid(**events_itr)==typeid(TextEvent::LocationY) ) {
        y = ((TextEvent::LocationY*)(*events_itr))->y;
    }
    else if ( typeid(**events_itr)==typeid(TextEvent::Font) ) {
        fontPath = ((TextEvent::Font*)(*events_itr))->fontPath;
    }
    else if ( typeid(**events_itr)==typeid(TextEvent::Size) ) {
        size = ((TextEvent::Size*)(*events_itr))->size;
    }
    else if ( typeid(**events_itr)==typeid(TextEvent::Color) ) {
        color = ((TextEvent::Color*)(*events_itr))->color;
    }
    else if ( typeid(**events_itr)==typeid(TextEvent::Style) ) {
        style = ((TextEvent::Style*)(*events_itr))->style;
    }
    else if ( typeid(**events_itr)==typeid(TextEvent::Wait) ) {
        timerCount = ((TextEvent::Wait*)(*events_itr))->count;
    }
    else if ( typeid(**events_itr)==typeid(TextEvent::String) ) {
        Face* face = Face::create();
        face->initFrame();
        face->setSurface(Library::createString(((TextEvent::String*)(*events_itr))->str.c_str(), fontPath.c_str(), size, color, style));
        face->setDistRect(x,y);
        mediator->addFace(face);
        faces.push_back(face);
    }
    else {
        cerr << "無効なテキストイベント(" << typeid(**events_itr).name() <<")です。" << endl;
    }

    events_itr++;
    return true;
}

void Text::destroy()
{
    list<Face*>::iterator i = faces.begin();
    while ( i !=faces.end() ) {
        SDL_FreeSurface((*i)->surface);
        (*i)->destroy();
        i = faces.erase(i);
    }
    GC<Text>::destroy(this);
}

void Text::destroy(SceneMediator* mediator)
{
    list<Face*>::iterator i = faces.begin();
    while ( i !=faces.end() ) {
        mediator->removeFace(*i);
        i++;
    }
 
    mediator->removeAction(this);
    destroy();
}

Text* Text::clone()
{
    Text* text = Text::create(NULL);
    text->x = x;
    text->y = y;
    text->size = size;
    text->style = style;
    text->timerCount = timerCount;
    text->color = color;

    list<TextEvent::Event*>::iterator itr = events->begin();
    while ( itr != events->end() ) {
        text->events->push_back((*itr)->clone());
        itr++;
    }

    return text;
}

void Text::init(SceneMediator* mediator)
{
    events_itr = events->begin();
    mediator->addAction(this);
}
