/*

ダイコン： STGスクリプトインタープリタは日本語XMLタグによるシューティングゲームのエンジンです。
Copyright (C) 2009 karasimiso

このプログラムはフリーソフトウェアです。あなたはこれを、フリーソフトウェア財団によって発行されたGNU 一般公衆利用許諾書(バージョン3か、 それ以降のバージョンのうちどれか)が定める条件の下で再頒布または改変することができます。

このプログラムは有用であることを願って頒布されますが、*全くの無保証 *です。商業可能性の保証や特定目的への適合性は、言外に示されたものも 含め、全く存在しません。詳しくはGNU 一般公衆利用許諾書をご覧ください。

あなたはこのプログラムと共に、GNU 一般公衆利用許諾書のコピーを一部受け取っているはずです。もし受け取っていなければ、<http://www.gnu.org/licenses/> をご覧ください。



Daikon: STG Script Interpreter is game engine for shooting game in Japanese XML tags.
Copyright (C) 2009 karasimiso

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __TEXT_H__INCLUDED__
#define __TEXT_H__INCLUDED__
#include "FlyingObject.h"
#include "Scene/ActionListener.h"

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <list>

class Face;
class SceneMediator;

namespace TextEvent {

    class Event {
    public:
        virtual TextEvent::Event* clone() = 0;
    };

    class Font : public Event
    {
    public:
        std::string fontPath;
        TextEvent::Event* clone();
    };

    class LocationX  : public Event
    {
    public:
        double x;
        TextEvent::Event* clone();
    };

    class LocationY  : public Event
    {
    public:
        double y;
        TextEvent::Event* clone();
    };


    class Color : public Event
    {
    public:
        SDL_Color color;
        TextEvent::Event* clone();
    };

    class Style : public Event
    {
    public: 
        int style;
        TextEvent::Event* clone();
    };

    class Size : public Event
    {
    public: 
        int size;
        TextEvent::Event* clone();
    };

    class Wait : public Event
    {
    public:
        int count;
        TextEvent::Event* clone();
    };

    class String : public Event
    {
    public:
        std::string str;
        TextEvent::Event* clone();
    };
};

class Text : public FlyingObject, public ActionListener
{
private:
    std::string fontPath;
    double x, y;
    int size, style;
    SDL_Color color;
    int timerCount;
    std::list<Face*> faces;
    std::list<TextEvent::Event*>* events;
    std::list<TextEvent::Event*>::iterator events_itr;

public:
 
    Text();
    ~Text();

    void init(SceneMediator* mediator);
    Text* clone();

    virtual bool action(SceneMediator* mediator);
    void destroy();
    void destroy(SceneMediator* mediator);

    static Text* create(std::list<TextEvent::Event*>* events);
};

#endif

